package hdm.mi.growbros.models.search;

import com.fasterxml.jackson.core.JacksonException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import hdm.mi.growbros.models.plant.*;
import hdm.mi.growbros.repositories.PlantRepository;
import org.junit.jupiter.api.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import java.util.List;

import static org.junit.jupiter.api.Assertions.*;

@SpringBootTest
class PlantSpecificationTest {

    @Autowired
    private PlantRepository plantRepository;
    @Autowired
    private PlantSpecification plantSpecification;

    private static Plant plant1;
    private static Plant plant2;
    private static Plant plant3;


    @BeforeAll
    public static void setUp(@Autowired PlantRepository plantRepository) {
        String jsonData = "[{\"name\":\"Etampes\",\"latinName\":\"ns12\",\"growingTips\":\"Wunderbar nutzbringende Untersaat z.B. für Grünkohl. Als Winterbegrünung\\nvon Staudenbeeten oder Balkonkästen.\",\"description\":\"Alte, dunkelgrüne, etwas langsamer wachsende Sorte, für den Freilandanbau. Erträgt Kahlfröste besser als andere Sorten und stängelt im Frühjahr etwas später auf. Löffelt nicht.\",\"origin\":null,\"plantWeekStart\":10,\"plantWeekEnd\":19,\"harvestWeekStart\":39,\"harvestWeekEnd\":15,\"growthDuration\":20,\"nutrientDemand\":\"LOW\",\"lightingDemand\":\"HIGH\",\"waterDemand\":\"DRY\",\"groundType\":\"HEAVY\",\"imageUrl\":\"https://d3t240q2dksnz7.cloudfront.net/7c982b256c33faf0068e58854d46efc1.jpg\"},{\"name\":\"Elan\",\"latinName\":\"ns15\",\"growingTips\":\"Wenn Feldsalat im heißen und trockenen August ausgesät wird kann es zu erheblichen Auflaufproblemen kommen. Hier helfen eine Beschattung mit einem Vlies und das Bewässern des Bodens vor der Aussaat. Feldsalat braucht wenig Stickstoff und sollte keinesfalls nach Hülsenfrüchten anbaut werden.\",\"description\":\"Sehr raschwachsende und mehltautolerante Sorte, die sich sowohl für den Anbau im Freien wie unter Glas und Folie eignet. Noch rascher im Wuchs als die Sorte Vit. Kompakte, glattblättrige Pflanzen mit hohem Einzelpflanzengewicht. Robust gegen falschen Mehltau. Neigt bei früher Aussaat zum Löffeln, daher nicht für die ersten Sätze im Sommer geeignet.  \",\"origin\":null,\"plantWeekStart\":10,\"plantWeekEnd\":19,\"harvestWeekStart\":37,\"harvestWeekEnd\":15,\"growthDuration\":18,\"nutrientDemand\":\"LOW\",\"lightingDemand\":\"HIGH\",\"waterDemand\":\"WET\",\"groundType\":null,\"imageUrl\":\"https://d3t240q2dksnz7.cloudfront.net/3b9ebf9fc01ef9b543a658efca59d56a.jpg\"},{\"name\":\"Postelein\",\"latinName\":\"Claytonia perfoliata L.\",\"growingTips\":\"Die Samen des Postelein keimen erst bei einer Temperatur unter 12 °C und werden darum in der Zeit von September bis März in Reihen mit 15–20 cm Abstand ausgesät (daher der Name \\\"Winterportulak\\\"). Meist stammt der im Handel erhältliche Postelein aus dem Gewächshausanbau. Auch wenn die Pflanzen etwas Kälte vertragen, solltest du sie mit einem Gartenvlies vor starkem Frost schützen. Während der Kultur ist der Postelein ziemlich anspruchslos. Zu Ernte schneidest du die (möglichst noch jungen) Blätter und Stiele nicht zu tief über dem Boden ab. So kann der Postelein noch einmal austreiben und es sind mehrere Ernten möglich.\\n\\n\",\"description\":\"Das \\\"Gewöhnliche Tellerkraut\\\" bzw. \\\"Postelein\\\" (Claytonia perfoliata) ist eine Pflanzenart aus der Familie der Quellkrautgewächse (Montiaceae). Sie wird in Mitteleuropa angebaut, ist winterhart und wird darum teilweise auch als \\\"modernes\\\" Wintergemüse verwendet.\",\"origin\":\"Die Berg- und Küstenregionen im Westen Nordamerikas\",\"plantWeekStart\":null,\"plantWeekEnd\":null,\"harvestWeekStart\":39,\"harvestWeekEnd\":20,\"growthDuration\":12,\"nutrientDemand\":\"HIGH\",\"lightingDemand\":\"LOW\",\"waterDemand\":\"WET\",\"groundType\":null,\"imageUrl\":\"https://d3t240q2dksnz7.cloudfront.net/b42b406101112ef7f194a4e9cb3afff2.jpg\"}]";

        try {
            final ObjectMapper objectMapper = new ObjectMapper();
            List<Plant> plants = objectMapper.readValue(jsonData, new TypeReference<>() {
            });
            plant1 = plants.get(0);
            plant2 = plants.get(1);
            plant3 = plants.get(2);
            plantRepository.saveAll(plants);
            System.out.println("plants saved");
        } catch (JacksonException e) {
            e.printStackTrace();
        }

    }

    @Test
    public void testNameExact() {
        SearchRequest searchRequest = SearchRequest.builder().searchTerm("Etampes").build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant1), plantRepository.findAll(spec));
    }

    @Test
    public void testNameIgnoreCase() {
        SearchRequest searchRequest = SearchRequest.builder().searchTerm("etampes").build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant1), plantRepository.findAll(spec));
    }

    @Test
    public void testNameIgnoreCase2() {
        SearchRequest searchRequest = SearchRequest.builder().searchTerm("eTaMpEs").build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant1), plantRepository.findAll(spec));
    }

    @Test
    public void testNameIncomplete() {
        SearchRequest searchRequest = SearchRequest.builder().searchTerm("ampe").build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant1), plantRepository.findAll(spec));
    }

    @Test
    public void testWaterDemand() {
        SearchRequest searchRequest = SearchRequest.builder().waterDemand(WaterDemand.WET).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant2, plant3), plantRepository.findAll(spec));
    }

    @Test
    public void testNutrientDemand() {
        SearchRequest searchRequest = SearchRequest.builder().nutrientDemand(NutrientDemand.LOW).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant1, plant2), plantRepository.findAll(spec));
    }

    @Test
    public void testLightingDemand() {
        SearchRequest searchRequest = SearchRequest.builder().lightingDemand(LightingDemand.HIGH).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant1, plant2), plantRepository.findAll(spec));
    }

    @Test
    public void testGroundType() {
        SearchRequest searchRequest = SearchRequest.builder().groundType(GroundType.HEAVY).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant1), plantRepository.findAll(spec));
    }


    @Test
    public void testPlantWeek1() {
        SearchRequest searchRequest = SearchRequest.builder().plantWeekStart(2).plantWeekEnd(10).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant1, plant2), plantRepository.findAll(spec));
    }

    @Test
    public void testPlantWeek2() {
        SearchRequest searchRequest = SearchRequest.builder().plantWeekStart(13).plantWeekEnd(15).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant1, plant2), plantRepository.findAll(spec));
    }

    @Test
    public void testPlantWeek3() {
        SearchRequest searchRequest = SearchRequest.builder().plantWeekStart(19).plantWeekEnd(33).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant1, plant2), plantRepository.findAll(spec));
    }

    @Test
    public void testGrowthDuration1() {
        SearchRequest searchRequest = SearchRequest.builder().growthDurationMin(11).growthDurationMax(20).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant1, plant2, plant3), plantRepository.findAll(spec));
    }

    @Test
    public void testGrowthDuration2() {
        SearchRequest searchRequest = SearchRequest.builder().growthDurationMin(20).growthDurationMax(50).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant1), plantRepository.findAll(spec));
    }

    @Test
    public void testGrowthDuration3() {
        SearchRequest searchRequest = SearchRequest.builder().growthDurationMin(1).growthDurationMax(19).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant2, plant3), plantRepository.findAll(spec));
    }

    @Test
    public void test_searchTerm_WaterDemand_1() {
        SearchRequest searchRequest = SearchRequest.builder().searchTerm("Postelein").waterDemand(WaterDemand.WET).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant3), plantRepository.findAll(spec));
    }

    @Test
    public void test_searchTerm_WaterDemand_2() {
        SearchRequest searchRequest = SearchRequest.builder().searchTerm("Salat").waterDemand(WaterDemand.WET).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(), plantRepository.findAll(spec));
    }

    @Test
    public void test_searchTerm_WaterDemand_3() {
        SearchRequest searchRequest = SearchRequest.builder().searchTerm("el").waterDemand(WaterDemand.WET).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant2, plant3), plantRepository.findAll(spec));
    }

    @Test
    public void test_WaterDemand_NutrientDemand_1() {
        SearchRequest searchRequest = SearchRequest.builder().waterDemand(WaterDemand.WET).nutrientDemand(NutrientDemand.LOW).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant2), plantRepository.findAll(spec));
    }


    @Test
    public void test_WaterDemand_NutrientDemand_2() {
        SearchRequest searchRequest = SearchRequest.builder().waterDemand(WaterDemand.WET).nutrientDemand(NutrientDemand.HIGH).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant3), plantRepository.findAll(spec));
    }

    @Test
    public void test_WaterDemand_NutrientDemand_3() {
        SearchRequest searchRequest = SearchRequest.builder().waterDemand(WaterDemand.DRY).nutrientDemand(NutrientDemand.HIGH).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(), plantRepository.findAll(spec));
    }

    @Test
    public void test_WaterDemand_NutrientDemand_groundType() {
        SearchRequest searchRequest = SearchRequest.builder().waterDemand(WaterDemand.DRY).nutrientDemand(NutrientDemand.LOW).groundType(GroundType.HEAVY).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant1), plantRepository.findAll(spec));
    }

    @Test
    public void test_WaterDemand_NutrientDemand_groundType_lightingDemand() {
        SearchRequest searchRequest = SearchRequest.builder().waterDemand(WaterDemand.DRY).nutrientDemand(NutrientDemand.LOW).groundType(GroundType.HEAVY).lightingDemand(LightingDemand.HIGH).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant1), plantRepository.findAll(spec));
    }

    @Test
    public void test_WaterDemand_NutrientDemand_groundType_lightingDemand_2() {
        SearchRequest searchRequest = SearchRequest.builder().waterDemand(WaterDemand.DRY).nutrientDemand(NutrientDemand.LOW).groundType(GroundType.HEAVY).lightingDemand(LightingDemand.LOW).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(), plantRepository.findAll(spec));
    }

    @Test
    public void test_NutrientDemand_growthDuration() {
        SearchRequest searchRequest = SearchRequest.builder().nutrientDemand(NutrientDemand.LOW).growthDurationMin(0).growthDurationMax(20).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant1, plant2), plantRepository.findAll(spec));
    }


    @Test
    public void test_NutrientDemand_growthDuration_2() {
        SearchRequest searchRequest = SearchRequest.builder().nutrientDemand(NutrientDemand.LOW).growthDurationMin(10).growthDurationMax(18).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant2), plantRepository.findAll(spec));
    }


    @Test
    public void test_NutrientDemand_growthDuration_3() {
        SearchRequest searchRequest = SearchRequest.builder().nutrientDemand(NutrientDemand.LOW).growthDurationMin(21).growthDurationMax(25).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(), plantRepository.findAll(spec));
    }


    @Test
    public void test_NutrientDemand_plantWeek_1() {
        SearchRequest searchRequest = SearchRequest.builder().nutrientDemand(NutrientDemand.LOW).plantWeekStart(19).plantWeekEnd(20).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant1, plant2), plantRepository.findAll(spec));
    }

    @Test
    public void test_NutrientDemand_plantWeek_2() {
        SearchRequest searchRequest = SearchRequest.builder().nutrientDemand(NutrientDemand.LOW).plantWeekStart(12).plantWeekEnd(20).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(plant1, plant2), plantRepository.findAll(spec));
    }

    @Test
    public void test_NutrientDemand_plantWeek_3() {
        SearchRequest searchRequest = SearchRequest.builder().nutrientDemand(NutrientDemand.LOW).plantWeekStart(20).plantWeekEnd(30).build();
        var spec = plantSpecification.withSearchCriteria(searchRequest);
        assertEquals(List.of(), plantRepository.findAll(spec));
    }

}
